﻿// The dollar sign ($) is an alias for jQuery that was passed into 
// the anonymous function making dollar sign safe to use.
(function ($) {
    // Module containing all the public functions for the plugin.
    ScrollableTable = function (element, options) {
        this.options = options;
        this.element = element;
        element.data("scrollableTable", this);

        this.init = function (element, options) {
            /// <summary>
            /// Initialize the plugin.
            /// </summary>
            /// <param name="element" type="jQuery Object">The 'root' element to create a scrollable table.</param>
            /// <param name="options" type="JSON Object">
            /// The options for the scrollable table.  Options are:
            /// 1. height (Integer): height of the table.
            /// 2. columns (Array): array of columns.  Each column has the following attributes:
            ///    - title (String): the column title.
            ///    - width (Integer): the column width.
            ///    - field (String): the attribute in the JSON object containing the value to be displayed in a table cell.
            ///    - isDate (Boolean): flag indicating that the value is a date.
            /// 3. data (Array): array of JSON objects containing the data to be displayed.
            /// </param>

            var that = this;
            // The "root" div is the scollable-table.
            element.addClass("scrollable-table");

            // layout of the scrollable table is:
            //  <div class='scrollable-table'>
            //      <div class='table-heading'>
            //          <table>
            //              <colgroup></colgroup>
            //              <thead>
            //                  <tr></tr>
            //              </thead>
            //          </table>
            //      </div>
            //      <div class='table-body'>
            //          <table>
            //              <colgroup></colgroup>
            //              <tbody></tbody>
            //          </table>
            //      </div>
            //  </div>

            // Create the elements for the table-heading.
            var $heading = $("<div class='table-heading'><table><colgroup></colgroup><thead><tr></tr></thead></table></div>");
            var $headingTableColGroup = $heading.find("colgroup");
            var $headingTableRow = $heading.find("tr");

            // Create the elements for the table-body.
            var $body = $("<div class='table-body' style='height:" + that.options.height + "px;'><table><colgroup></colgroup><tbody></tbody></table></div>");
            var $bodyTableColGroup = $body.find("colgroup");
            var $bodyTableTbody = $body.find("tbody");

            // In the table-heading and table-body, add a col to the colgroup for each column.
            // In the table-heading, add a table cell with the title of each column.
            $.each(that.options.columns, function (idx, column) {
                $headingTableColGroup.append("<col style='width:" + column.width + "px;'/>");
                $bodyTableColGroup.append("<col style='min-width:" + column.width + "px; width: " + ((idx == that.options.columns.length - 1) ? "100%" : column.width + "px") + ";'/>");
                $headingTableRow.append("<td>" + column.title + "</td>");
            });

            // Add the table-heading to the scrollable-table.
            element.append($heading[0].outerHTML);

            this._displayData($bodyTableTbody, that.options.data);

            // Add the table-body to the scrollable-table.
            element.append($body[0].outerHTML);
        };

        this.applyFilter = function (criteria) {
            /// <summary>
            /// Filter the data in the table.
            /// </summary>
            /// <param name="criteria" type="JSON object">
            /// The JSON object has 2 attributes:
            /// 1. field: the field to filter on.
            /// 2. exp: the regular expression to apply to the field.
            /// </param>

            var that = this;
            setTimeout(function () {
                var filtered = [];

                $.each(that.options.data, function (idx, item) {
                    if (criteria.exp.test(item[criteria.field]) == true) {
                        filtered.push(item);
                    }
                });

                $tbody = that.element.find(".table-body tbody");
                $tbody.empty();

                that._displayData($tbody, filtered);
                that.filtered = filtered;
            });
        };

        this.removeFilter = function () {
            /// <summary>
            /// Remove the filter.
            /// </summary>

            var that = this;
            setTimeout(function () {
                $tbody = that.element.find(".table-body tbody");
                $tbody.empty();
                that._displayData($tbody, that.options.data);
                that.filtered = null;
            });
        };

        this._displayData = function ($tbody, data) {
            /// <summary>
            /// Display the data in the table body.
            /// </summary>

            var that = this;

            // In the table-body, add a row for each data item.
            $.each(data, function (idx, item) {
                $tableRow = $("<tr></tr>");

                // For each column in the data item, add a table cell with the value.
                $.each(that.options.columns, function (col, column) {
                    $tableRow.append("<td>" +
                        (column.isDate
                            ? item[column.field].toDateString()
                            : item[column.field]) +
                        "</td>");
                });

                // Add the row to the table-body.
                $tbody.append($tableRow[0].outerHTML);
            });
        };

        // Initialize the scrollable table.
        this.init(element, options);
    };

    // The Scrollable Table jQuery plugin.
    $.fn.scrollableTable = function (options) {
        return this.each(function () {
            // Instantiate the module and pass in a reference to the element and the options passed in.
            (new ScrollableTable($(this), options));
        });
    };
})(jQuery); // Pass jQuery as a parameter into the anonymous function.